﻿using CsvHelper;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BolshakovExam
{
    internal class Program
    {
        static void Main(string[] args)
        {
            Debug.Listeners.Add(new TextWriterTraceListener("Debug.txt"));
            Debug.AutoFlush = true;
            Debug.Indent();

            Trace.Listeners.Add(new TextWriterTraceListener("Trace.txt"));
            Trace.AutoFlush = true;
            Trace.Indent();

            int[] n = Reading("data.csv").Item1;
            int[] m = Reading("data.csv").Item2;
            double[,] costs = Reading("data.csv").Item3;

            MinElement min = new MinElement();
            
            int[,] supply_plan = min.MinElemDecision(n, m, costs);
            Writing(supply_plan);

        }

        // Метод для считывания данных из файла
        private static (int[], int[], double[,]) Reading(string path)
        {
            if (!path.Contains(".csv"))
            {
                int[] n = new int[0];
                int[] m = new int[0];
                double[,] costs = new double[0,0];
                Console.WriteLine("File Error!!!");
                Console.ReadKey();
                return (n, m, costs);
            }
            else
            {
                var dataStr = File.ReadAllLines(path).Select(line => line.Split(';').ToArray()).ToArray();

                int[] n = new int[dataStr[0].Length];
                int[] m = new int[dataStr[1].Length];
                double[,] costs = new double[dataStr.Length - 2, dataStr[0].Length];

                for (int i = 0; i < dataStr.Length; i++)
                {
                    for (int j = 0; j < dataStr[i].GetLength(0); j++) 
                    {
                        if(i == 0)
                        {
                            n[j] = Convert.ToInt32(dataStr[i][j]);
                        }
                        else if (i == 1)
                        {
                            m[j] = Convert.ToInt32(dataStr[i][j]);
                        }
                    }
                }

                for (int i = 0; i < dataStr.Length - 2; i++) 
                {
                    for (int j = 0; j < dataStr[i].Length; j++) 
                    {
                        costs[i,j] = Convert.ToDouble(dataStr[i][j]);
                    }
                }

                return (n,m,costs);
            }
        }

        // Метод для записи решения в файл
        private static void Writing(int[,] supply_plan)
        {
            using (StreamWriter writer = new StreamWriter("result.csv"))
            {
                using (CsvWriter csv = new CsvWriter(writer, System.Globalization.CultureInfo.InvariantCulture))
                {
                    for (int i = 0; i < supply_plan.GetLength(0); i++) 
                    {
                        for(int j = 0; j < supply_plan.GetLength(1); j++)
                        {
                            csv.WriteRecord(supply_plan[i, j]);
                        }
                        csv.NextRecord();
                    }
                }
            }
        }

    }
    public class MinElement
    {
        // Метод для получения информации была заполнина или затронута(true) клетка или нет(false)
        private bool CheckMatrix(bool[,] matrix)
        {
            for(int i = 0; i < matrix.GetLength(0); i++)
            {
                for (int j = 0; j < matrix.GetLength(1); j++)
                {
                    if (matrix[i,j] == false)
                    {
                        return false;
                    }
                }
            }
            return true;
        }

        // Метод для нахождения минимальных элементов в матрице
        private List<(int,int)> FindMinElemI(double[,] costs, bool[,] matrix)
        {
            List<(int,int)> position = new List<(int,int)> ();   // Лист для хранения индексов минимального тарифа
            double min = double.MaxValue;

            
            // Цикл для заполнения листа инлексами минимальных тарифов 
            for (int i = 0;i < costs.GetLength(0); i++)
            {
                for (int j = 0; j < costs.GetLength(1); j++)
                {
                    if (matrix[i, j] == false && costs[i,j] < min)
                    {
                        min = costs[i,j];
                        position.Clear();
                        position.Add((i, j));
                    }
                    else if(matrix[i,j] == false && costs[i,j] == min)
                    {
                        position.Add((i, j));
                    }
                }
            }
            return position;
        }
        
        // Основной метод решения
        public int[,] MinElemDecision(int[] a, int[] b, double[,] costs)
        {
            try
            {
                // Перепменные для первоначальных данных
                int n = a.Length;
                int m = b.Length;
                int[,] supply_plan = new int[n, m];
                bool[,] bool_supply_plan = new bool[n, m];

                // Цикл пока вся матрица не будет true
                while (!CheckMatrix(bool_supply_plan))
                {
                    // Лист для получения индексов минимальных тарифов
                    List<(int, int)> position = FindMinElemI(costs, bool_supply_plan);
                    int min_j = int.MaxValue;
                    int minIndex = -2;

                    for (int k = 0; k < position.Count; k++)
                    {
                        if (position[k].Item2 < min_j)
                        {
                            min_j = position[k].Item2;
                            minIndex = k;
                        }
                    }

                    int min_i = position.ElementAt(minIndex).Item1;

                    int value = Math.Min(a[min_i], b[min_j]);

                    a[min_i] -= value;
                    b[min_j] -= value;
                    supply_plan[min_i, min_j] = value;
                    bool_supply_plan[min_i, min_j] = true;

                    Debug.WriteLine($"Промежуточный подсчёт a[min_i]: {a[min_i]} Промежуточный подсчёт b[min_j]: {b[min_j]}");
                    Trace.WriteLine($"Промежуточный подсчёт a[min_i]: {a[min_i]} Промежуточный подсчёт b[min_j]: {b[min_j]}");

                    // Заполнение матрици true елси значние поставщика равна 0
                    if (a[min_i] == 0)
                    {
                        for (int j = 0; j < m; j++)
                        {
                            if (bool_supply_plan[min_i, j] == false)
                            {
                                bool_supply_plan[min_i, j] = true;
                            }
                        }
                    }

                    // Заполнение матрици true елси значние потребителя равна 0
                    if (b[min_j] == 0)
                    {
                        for (int i = 0; i < n; i++)
                        {
                            if (bool_supply_plan[i, min_j] == false)
                            {
                                bool_supply_plan[i, min_j] = true;
                            }
                        }
                    }
                }
                return supply_plan;
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                return null;
            }

        }
    }
}
